import os

from icometrix_sdk import IcometrixApi
from icometrix_sdk.models.upload_entity import StartUploadDto

PROJECT_ID = "uuid"
DICOM_DIR_PATH = "<path>"

if __name__ == '__main__':
    os.environ["API_HOST"] = "https://icobrain-test.icometrix.com"

    # Initialize the icometrix API
    ico_api = IcometrixApi()

    # Get the project, to make sure its there (will throw a 404 in case the project is not found)
    project = ico_api.projects.get_one_by_id(PROJECT_ID)

    # Upload a directory of DICOMs
    data = StartUploadDto(icobrain_report_type="icobrain_ms")
    upload = ico_api.uploads.start_upload(PROJECT_ID, data)

    # Loop over all files in a DIR and upload them to the before created upload entry
    for path, subdirs, files in os.walk(DICOM_DIR_PATH):
        for name in files:
            if name.startswith("."):
                continue
            ico_api.uploads.upload_dicom_path(upload.uri, os.path.join(path, name))

    file_list = ico_api.uploads.get_uploaded_files(upload.folder_uri)



class IcometrixException(Exception):
    pass


class IcometrixHttpException(IcometrixException):
    fmt = "An HTTP Client raised an unhandled exception: {error}"


class IcometrixParseException(IcometrixException):
    fmt = "Failed to parse HTTP response body"


class IcometrixConfigException(IcometrixException):
    fmt = "Invalid configuration"


class IcometrixAuthException(IcometrixException):
    fmt = "Authentication failed"



from typing import Optional, Union

from icometrix_sdk.models.base import BackendEntity


class CustomerReportEntity(BackendEntity):
    cost: Optional[Union[str, float]] = None
    status: str
    study_id: str
    patient_id: str
    project_id: str
    study_date: Optional[str] = None
    report_type: Optional[str] = None
    patient_name: Optional[str] = None
    project_name: str
    report_files: list
    report_status: Optional[str] = None
    project_country: str
    accession_number: Optional[str] = None
    dicom_patient_id: Optional[str] = None
    customer_result_id: Optional[str] = None
    pipeline_result_id: Optional[str] = None
    study_instance_uid: str
    icobrain_report_type: str
    report_translated_remarks: Optional[str] = None



from pydantic import BaseModel
from typing import Generic, TypeVar, List, Optional, Iterator

T = TypeVar("T")


class BackendEntity(BaseModel):
    id: str
    uri: Optional[str] = None
    update_timestamp: str
    creation_timestamp: str


class PaginatedResultSet(BaseModel):
    count: int
    offset: int
    limit: int


class PaginatedMetaData(BaseModel):
    result_set: PaginatedResultSet


class PaginatedResponse(BaseModel, Generic[T]):
    meta_data: PaginatedMetaData
    results: List[T]
    _itr_index: int

    def has_next(self) -> bool:
        """
        Returns True if there is a next page in the backend

        :return: bool
        """
        result_set = self.meta_data.result_set
        return int(result_set.count) > (int(result_set.offset) + int(result_set.limit))

    def __iter__(self) -> Iterator[T]:
        return iter(self.results)

    def __getitem__(self, index) -> T:
        return self.results[index]

    def __next__(self) -> T:
        if len(self.results) < self._itr_index:
            raise StopIteration

        item: T = self.results[self._itr_index]
        self._itr_index += 1
        return item

    def __len__(self) -> int:
        return len(self.results)



from typing import Optional, List, Union

from pydantic import BaseModel

from icometrix_sdk.models.base import BackendEntity


class StartUploadDto(BaseModel):
    icobrain_report_type: str


class UploadEntity(BackendEntity):
    status: str
    folder_uri: str
    files_uri: List[str]
    type: str
    compressed: Optional[Union[str, bool]] = None
    logs: List[str]
    errors: List
    icobrain_report_type: str
    retry_count: Optional[Union[str, int]] = None
    id: str
    project_id: str
    creation_timestamp: str
    update_timestamp: str
    file_name: str
    uri: str


class UploadEntityFiles(BaseModel):
    files: List[str]



"""A library that provides a Python interface to the Icometrix API"""
import os
from typing import Optional

from icometrix_sdk.authentication import PasswordAuthentication, AuthenticationMethod, get_auth_method
from icometrix_sdk.exceptions import IcometrixConfigException
from icometrix_sdk.resources.customer_reports import CustomerReports
from icometrix_sdk.models.base import PaginatedResponse
from icometrix_sdk.resources.patients import Patients
from icometrix_sdk.resources.profile import Profile
from icometrix_sdk.resources.projects import Projects
from icometrix_sdk.resources.uploads import Uploads
from icometrix_sdk.utils.requests_api_client import RequestsApiClient
from icometrix_sdk.utils.api_client import ApiClient


def get_api_client() -> RequestsApiClient:
    """
    Create an RequestsApiClient with the 'API_HOST' environment variable as server
    """
    api_host = os.getenv("API_HOST")
    if not api_host:
        raise IcometrixConfigException("Please set the 'API_HOST' environment variable")

    auth_method = get_auth_method()
    return RequestsApiClient(api_host, auth_method)


class IcometrixApi:
    """
    A Python interface for the icometrix API.

    :param api_client:
        An instance of an ApiClient
    """

    profile: Profile
    projects: Projects
    patients: Patients
    uploads: Uploads
    customer_reports: CustomerReports

    _api_client: ApiClient

    def __init__(self, api_client: Optional[ApiClient] = None):
        self._api_client = api_client
        if not self._api_client:
            self._api_client = get_api_client()

        self.profile = Profile(self._api_client)
        self.projects = Projects(self._api_client)
        self.patients = Patients(self._api_client)
        self.uploads = Uploads(self._api_client)
        self.customer_reports = CustomerReports(self._api_client)

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        pass



import os

from icometrix_sdk import IcometrixApi
from icometrix_sdk.models.upload_entity import StartUploadDto

PROJECT_ID = "uuid"
DICOM_DIR_PATH = "<path>"
DICOM_PATHS = [
    "IM-0001-0001.dcm"
    "IM-0001-0002.dcm"
]

if __name__ == '__main__':
    os.environ["API_HOST"] = "https://icobrain-test.icometrix.com"

    # Initialize the icometrix API
    ico_api = IcometrixApi()

    # Get the project, to make sure its there (will throw a 404 in case the project is not found)
    project = ico_api.projects.get_one_by_id(PROJECT_ID)

    # prepare an upload entry, this entry can be used to push a block of files to (which is typical in DICOM)
    # Some options can be passed, for all available options please have a look at the StartUploadDto
    data = StartUploadDto(icobrain_report_type="icobrain_ms")
    upload = ico_api.uploads.start_upload(PROJECT_ID, data)

    # Loop over all files and upload them to the before created upload entry
    for dicom_path in DICOM_PATHS:
        ico_api.uploads.upload_dicom_path(upload.uri, dicom_path)

    # Once all files have been uploaded, signal that they are all there and start the import/processing
    upload = ico_api.uploads.complete_upload(upload.uri)



import os

from icometrix_sdk import IcometrixApi
from icometrix_sdk.models.upload_entity import StartUploadDto

PROJECT_ID = "uuid"
DICOM_DIR_PATH = "<dir_path>"

if __name__ == '__main__':
    os.environ["API_HOST"] = "https://icobrain-test.icometrix.com"

    # Initialize the icometrix API
    ico_api = IcometrixApi()

    # Get the project, to make sure its there (will throw a 404 in case the project is not found)
    project = ico_api.projects.get_one_by_id(PROJECT_ID)

    # Upload a directory of DICOMs
    data = StartUploadDto(icobrain_report_type="icobrain_ms")
    ico_api.uploads.upload_dicom_dir(PROJECT_ID, DICOM_DIR_PATH, data)



from icometrix_sdk.models.base import PaginatedResponse
from icometrix_sdk.models.customer_report_entity import CustomerReportEntity
from icometrix_sdk.utils.requests_api_client import ApiClient


class CustomerReports:
    def __init__(self, api: ApiClient):
        self._api = api

    def get_all(self, project_id: str, **kwargs) -> PaginatedResponse[CustomerReportEntity]:
        """
        Get al customer reports for a project

        :param project_id: The ID of the project you want to fetch the customer-reports from
        :return: A Paginated response containing customer-reports
        """

        page = self._api.get(f"/customer-reports-service/api/v2/projects/{project_id}/customer-reports", **kwargs)
        return PaginatedResponse[CustomerReportEntity](**page)

    def get_one(self, customer_report_uri: str) -> CustomerReportEntity:
        """
        Get a single customer-report based on the customer-report uri

        :param customer_report_uri: the uri of the customer-report
        :return: A single customer-report or 404
        """
        resp = self._api.get(customer_report_uri)
        return CustomerReportEntity(**resp)

    def delete(self, customer_report_uri: str, **kwargs):
        """
        Delete a customer-report

        :param customer_report_uri: The uri of the customer-resport you want to delete
        :return:
        """

        self._api.delete(customer_report_uri, **kwargs)



import json
import logging
import os
import threading
import uuid
from functools import lru_cache
from typing import Optional, Type, Dict
from urllib.parse import urljoin

import requests
from requests import Response, HTTPError, Session
from requests.adapters import HTTPAdapter
from urllib3.util import Retry

from icometrix_sdk._version import __version__
from icometrix_sdk.authentication import AuthenticationMethod
from icometrix_sdk.exceptions import IcometrixHttpException, IcometrixParseException, IcometrixConfigException, \
    IcometrixAuthException
from icometrix_sdk.utils.api_client import ApiClient
from icometrix_sdk.utils.file_upload import create_multipart

HTTP_TIMEOUT = os.getenv("HTTP_TIMEOUT", 120)

logger = logging.getLogger(__name__)


class RequestsApiClient(ApiClient):
    """
    An implementation of the API client using the requests library

    Dev Note: Try not the expose requests logic, we might want to swap this later on
    """

    _auth_attempts = 0

    _session: Session
    _auth: Optional[AuthenticationMethod]

    def __init__(self, server: str, auth: Optional[AuthenticationMethod] = None):
        self.base_headers = {
            "User-Agent": "Python SDK v{}".format(__version__),
            "x-sdk-type": "Python",
            "x-sdk-version": __version__,
            "x-sdk-contextId": str(uuid.uuid4()),
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        self.server = server
        self._auth = auth

        if not server:
            raise IcometrixConfigException("Server is required")

        self._session = self._create_session()
        self._authenticate()

    def _authenticate(self):
        if self._auth:
            logger.info("Authenticating")
            self._auth_attempts += 1
            self._auth.connect(self)
            self._auth_attempts = 0

    def refresh_token(self, resp: Response):
        if self._auth_attempts > 0:
            raise IcometrixAuthException("Failed to authenticate")
        if resp.status_code == 401:
            logger.info("Fetching new token as the previous token expired")
            self._authenticate()

    def _create_session(self) -> Session:
        """
        Create a new session
        """
        session = requests.Session()
        retries = Retry(
            total=3,
            backoff_factor=0.5,
            status_forcelist=[429, 502, 503, 504],
        )
        session.mount('https://', HTTPAdapter(max_retries=retries))
        session.headers.update(self.base_headers)
        return session

    def _make_request(self,
                      method=Type[str],
                      url=Type[str],
                      data: Optional[any] = None,
                      headers: Optional[dict] = None,
                      params: Optional[dict] = None,
                      files=None,
                      cookies=None) -> Response:
        """
        private function to abstract most of the HTTP request logic

        :param method: GET, POST, PUT, PATCH, DELETE...
        :param url: A relative URL to specify the API endpoint
        :param data: Dict containing data for the request body
        :param headers: Dict containing additional request headers.
        :param params: Dict containing query parameters.
        :param files: (optional) Dict of ``'filename': file-like-objects`` for multipart encoding upload.
        :param cookies: (optional) Dict or CookieJar object to send with the
        :return:
        """

        full_url = urljoin(self.server, url)
        try:
            response = self._session.request(
                method=method,
                url=full_url,
                data=data,
                headers=headers,
                params=params,
                files=files,
                timeout=HTTP_TIMEOUT,
                cookies=cookies
            )
            response.raise_for_status()
        except HTTPError as e:
            message = "Exception received when sending an HTTP request"
            logger.exception(message)
            raise IcometrixHttpException(e)

        return response

    def get(self, uri: str, params: Optional[dict] = None, **kwargs) -> dict:
        """
        Submit a GET request to the API.

        :param uri: A relative URL to specify the API endpoint
        :param params: A dictionary containing query parameters.
        :returns: The API response as a dictionary.
        """

        resp = self._make_request(
            method="GET",
            url=uri,
            params=params,
            **kwargs
        )
        return response_to_dict(resp)

    def post(self, uri: str, data: dict, headers: Optional[dict] = None, **kwargs) -> dict:
        """
        Submit a POST request to the API.

        :param uri: A relative URL to specify the API endpoint
        :param data: A dictionary containing data for the request body
        :param headers: A dictionary containing additional request headers.
        :returns: The API response as a dictionary.

        """

        resp = self._make_request(
            method="POST",
            url=uri,
            data=json.dumps(data),
            headers=headers,
            **kwargs
        )
        return response_to_dict(resp)

    def put(self, uri: str, data: dict, **kwargs) -> dict:
        """
        Submit a PUT request to the API.

        :param uri: A relative URL to specify the API endpoint
        :param data: A dictionary containing data for the request body
        :returns: The API response as a dictionary.

        """

        resp = self._make_request(
            method="PUT",
            url=uri,
            data=json.dumps(data),
            **kwargs
        )
        return response_to_dict(resp)

    def delete(self, uri: str, **kwargs):
        """
        Submit a DELETE request to the API.

        :param uri: A relative URL to specify the API endpoint
        :returns: void

        """
        self._make_request(
            method="DELETE",
            url=uri,
            **kwargs
        )

    def put_file(self, uri: str, fields, **kwargs):
        """
        Submit a PUT request to the API.

        :param uri: A relative URL to specify the API endpoint
        :param fields: Dictionary of fields or list of (key, :class:`~urllib3.fields.RequestField`).
        :returns:

        """

        data, headers = create_multipart(fields)
        self._make_request(
            method="PUT",
            url=uri,
            data=data,
            headers=headers,
            **kwargs
        )


def response_to_dict(response: Response) -> dict:
    content = response.content
    try:
        return json.loads(content)
    except ValueError as e:
        raise IcometrixParseException("Failed to parse response to json: {}".format(e.args[0]))



import pydicom
from datetime import date, timedelta
from typing import Final, List, Tuple, Union

from icometrix_sdk.utils.hash_factory import HashMethod


class Anonymizer:
    """
    The anonymizer anonymizes DICOMs by:
    (1) replacing certain fields with a hash of the fields (such as UIDs)
    (2) shifting dates so that the birthday is alwasy set to the 1st of january of the birth year
    (3) removing all fields that are not anonymized or explicitly marked as to be kept.
    """

    # CONSTS
    BIRTHDAY_TAG: int = 0x00100030
    AGE_TAG: int = 0x00101010
    PATIENT_IDENTITY_REMOVED_TAG: int = 0x00120062
    DEIDENTIFICATION_METHOD_TAG: int = 0x00120063

    '''
    DICOM makes extensive use of Unique Identifiers. Almost every entity in the DICOM Data Model 
    has a unique identifier. In DICOM every SOP Class have its UID. All pre-defined UID's including 
    the SOP Class UID's are documented in chapter 6 of the DICOM standard. A DICOM Object is an 
    Instance of such class and is called SOP Instance and it also has a UID called SOP Instance UID.
    DICOM defines a mechanism in order to make sure UID's are globally Unique:

    Every DICOM application should acquire a "root" UID that is used as a prefix to the UID's it 
    creates.

    See https://dicomiseasy.blogspot.com/2011/12/chapter-4-dicom-objects-in-chapter-3.html
    '''
    ROOT_UID: Final[str] = "1.2.826.0.1.3680043.9.5542"

    def __init__(self, hash_method: HashMethod, override_whitelist_tags: List[int] = None,
                 override_whitelist_groups: List[int] = None,
                 override_whitelist_ranges: List[Tuple[int, int]] = None):

        self.hash_method = hash_method

        # Maximum age of a patient, because our population database only goes up to 90
        self.maximum_age = 90

        self.hash_uid_tags = [
            0x0020000d,  # StudyInstanceUID
            0x00200052,  # FrameOfReferenceUID
            0x00200200,  # SynchronizationFrameOfReferenceUID
            0x00080018,  # SOPInstanceUID
            0x00080014,  # InstanceCreatorUID
            0x00081155,  # RefSOPInstanceUID
            0x0020000e,  # SeriesInstanceUID
            0x0040a124,  # UID
            0x0070031a,  # FiducialUID
            0x00880140,  # StorageMediaFilesetUID
            0x30060024,  # ReferencedFrameOfReferenceUID
            0x300600c2,  # RelatedFrameOfReferenceUID
            0x300a0013  # DoseReferenceUID
        ]

        self.hash_name_tags = [
            0x00100010,  # PatientName
            0x00080090  # ReferringPhysicianName
        ]

        self.hash_tags = [
            0x00080050,  # AccessionNumber
            0x00080080,  # InstitutionName
            0x00100020,  # PatientId
            0x00100021,  # IssuerOfPatientID
            0x00101000,  # OtherPatientIDs
            0x00200010  # StudyID
        ]

        self.date_tags = [
            0x00080013,  # InstanceCreationDate
            0x00080020,  # StudyDate
            0x00080021,  # SeriesDate
            0x00080022,  # AcquisitionDate
            0x00080023,  # ContentDate
        ]

        self.time_tags = [
            0x00080013,  # InstanceCreationTime
            0x00080030,  # StudyTime
            0x00080031,  # SeriesTime
            0x00080032,  # AcquisitionTime,
            0x00180080,  # RepetitionTime
            0x00180081,  # EchoTime
            0x00180082  # InversionTime
        ]

        self.additional_whitelist_tag = [
            self.BIRTHDAY_TAG,
            0x00100040,  # PatientsSex
            0x00020000,  # MetaElementGroupLength
            0x00020001,  # FileMetaInfoVersion
            0x00020002,  # MediaStorageSOPClassUID
            0x00020003,  # MediaStorageSOPInstanceUID
            0x00020010,  # TransferSyntaxUID
            0x00020012,  # ImplementationClassUID
            # 0x00097772,  # CallingAET
        ] if override_whitelist_tags is None else override_whitelist_tags

        self.whitelisted_groups = [
            0x0008,  # Identifying information
            0x0015,  # icometrix private tags
            0x0018,  # Acquisition: mage acquisition device and imaging procedure
            0x0020,  # Relationship: Info relating the image to the location within the patient
            0x0028,  # Image presentation: manner in which the image can be presented or displayed
            0x0029,  # Groups specific to diffusion (b-vector and b-value, ...) Requested by Thibo
            0x5200,  # Multi-frame Functional Groups
        ] if override_whitelist_groups is None else override_whitelist_groups

        self.whitelisted_group_ranges = [
            (0x7f00, 0x7fff),  # Image pixel data
            (0xfff0, 0xffff)  # Needed to check after every SQ
        ] if override_whitelist_ranges is None else override_whitelist_ranges

    def anonymize(self, dicom: pydicom.FileDataset) -> pydicom.FileDataset:
        """
        Anonymizes a single FileDataSet by only keeping whitelisted tags, and hashing the necessary
        ones.

        :param dicom: dicom (pydicom.FileDataset): DICOM FileDataSet to be anonymized
        :return:  pydicom.FileDataset: Anonymized FileDataSet
        """

        anonymized_dicom = dicom.copy()

        time_shift = self._calculate_time_shift(dicom)

        for data_element in anonymized_dicom:

            tag = data_element.tag
            tag_value = data_element.value

            if tag_value is None:
                continue

            if not self._is_tag_whitelisted(tag):
                # Unsigned Short or Unsigned Long => should be numeric value
                if data_element.VR in ("US", "UL"):
                    anonymized_dicom[tag] = self._build_dataelem(data_element, 0)
                else:
                    anonymized_dicom[tag] = self._build_dataelem(data_element, "")
            elif tag in self.hash_uid_tags:
                anonymized_dicom[tag] = self._build_dataelem(
                    data_element, self.hash_uid(str(tag_value)))
            elif tag in self.hash_tags:
                anonymized_dicom[tag] = self._hash_dataelement(data_element)
            elif tag in self.hash_name_tags:
                anonymized_dicom[tag] = self._hash_dataelement(data_element)
            elif tag in self.time_tags:
                anonymized_dicom[tag] = self._build_dataelem(data_element, "010101")
            elif tag in self.date_tags:
                date_str = str(tag_value)
                orig_date = date(int(date_str[:4]), int(date_str[4:6]), int(date_str[6:8]))
                shifted_date = orig_date - time_shift
                anonymized_dicom[tag] = self._build_dataelem(
                    data_element,
                    f"{shifted_date.year}{shifted_date.month:02d}{shifted_date.day:02d}")

        # Field "PatientIdentityRemoved"
        # See https://dicom.innolitics.com/ciods/rt-plan/patient/00120062
        anonymized_dicom.add_new(self.PATIENT_IDENTITY_REMOVED_TAG, "LT", "YES")

        # Field "Deidentification method"
        # See https://dicom.innolitics.com/ciods/rt-plan/patient/00120063
        anonymized_dicom.add_new(self.DEIDENTIFICATION_METHOD_TAG, "LT",
                                 "Python based, HIPAA compliant, based on DICOM PS3.15 AnnexE.")

        return anonymized_dicom

    def _hash(self, obj: str, max_len=0) -> str:
        output = self.hash_method.calculate_hash(obj)

        if max_len and len(output) > max_len:
            output = output[:max_len]

        return output

    def hash_id(self, id_str: str, max_len=64):
        """

        :param id_str: id that needs to be hashed
        :param max_len: max length of the hash, i.e. 64 characters for LONG STRING
        :return: dicom-compliant hashed id, i.e., the hash of the ID with an even length
        """
        hashed_id = self._hash(id_str, max_len=max_len)

        # If value length is uneven, make it even
        if len(hashed_id) % 2 != 0:
            hashed_id = hashed_id[:-1]

        return hashed_id

    def hash_uid(self, uid: str, max_len=64, trim=False) -> str:

        # In icobridge code, sometimes only the last part of the uid is hashed
        # So this is an option here for backwards compatability
        to_hash = uid.split('.')[-1] if trim else uid

        output = self._hash(to_hash)

        # Convert hash in hexadecimal format to decimal format
        # Requirement of DICOM UIDs that they only exist of digits
        # See https://dicom.nema.org/dicom/2013/output/chtml/part05/chapter_9.html
        output = str(int(output, base=16))

        # Dicom 9.1: The first digit of each component shall not be zero,
        # unless the component is a single digit.
        # See https://dicom.nema.org/dicom/2013/output/chtml/part05/chapter_9.html
        extra = ""
        if output[0] == "0":
            extra = "9"

        output = f"{self.ROOT_UID}.{extra}{output}"

        # DICOM 9.1: UIDs, shall not exceed 64 total characters
        if len(output) > max_len:
            output = output[:max_len]

        return output

    def _hash_dataelement(self, data_elem: pydicom.DataElement) -> pydicom.DataElement:
        max_len = 0
        # Short string has a max length of 16
        if data_elem.VR == "SH":
            max_len = 16
        value_hash = self._hash(data_elem.value, max_len=max_len)
        return pydicom.DataElement(VR=data_elem.VR, tag=data_elem.tag, value=value_hash)

    def _is_tag_whitelisted(self, tag) -> bool:

        # Concatenated version of all tags: combine lists and flatten them
        whitelisted_tags = [tag for sublist in
                            [self.hash_uid_tags, self.hash_name_tags, self.hash_tags,
                             self.time_tags, self.date_tags, self.additional_whitelist_tag]
                            for tag in sublist]

        if tag in whitelisted_tags:
            return True

        for group in self.whitelisted_groups:
            lower_bound = group * 0x10000
            upper_bound = group * 0x10000 + 0xffff
            if lower_bound <= tag <= upper_bound:
                return True

        for group_range in self.whitelisted_group_ranges:
            lower_range_bound = group_range[0]
            upper_range_bound = group_range[1]
            lower_bound = lower_range_bound * 0x10000
            upper_bound = upper_range_bound * 0x10000 + 0xffff

            if lower_bound <= tag <= upper_bound:
                return True

        return False

    @staticmethod
    def _build_dataelem(data_elem: pydicom.DataElement, value: Union[str, int]) \
            -> pydicom.DataElement:
        return pydicom.DataElement(VR=data_elem.VR, value=value, tag=data_elem.tag)

    def _calculate_time_shift(self, dicom: pydicom.FileDataset) -> timedelta:
        """
        Return the appropriate time shift. This is defined as the difference between
        the birthdate of the patient and the first of january. If no birthdate is given,
        and instead an age is given, the time shift is 0.
        """

        birthday_dataelem = dicom[self.BIRTHDAY_TAG]

        # Calculate anchor date: the date on which the time shift is based
        # First, check if we have the necessary data
        if birthday_dataelem is None:
            return timedelta(days=0)

        birthday = birthday_dataelem.value
        if not isinstance(birthday, str):
            return timedelta(days=0)

        # Birthday is set, so we take (month, day) to calculate the time shift
        if len(birthday) != 8:
            return timedelta(days=0)
        anchor_date = date(int(birthday[:4]), int(birthday[4:6]), int(birthday[6:8]))

        january_1 = date(anchor_date.year, 1, 1)
        return anchor_date - january_1



import logging
import os

from icometrix_sdk import IcometrixApi
from icometrix_sdk.utils.paginator import get_paginator

logging.basicConfig(level=logging.INFO)

if __name__ == '__main__':

    os.environ["API_HOST"] = "https://icobrain-test.icometrix.com"

    with IcometrixApi() as ico_api:
        for projects in get_paginator(ico_api.projects.get_all):
            for project in projects:
                print(project.name)
